# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import json
from pathlib import Path

import click
from packaging.version import parse
from jupyter_releaser.util import get_version, run


LERNA_CMD = "jlpm run lerna version --no-push --force-publish --no-git-tag-version"


def increment_version(current: str, spec: str):
    """
    current: current version
    spec: how to increment the version
    
    """
    curr = parse(current)

    if spec == "major":
        return f"{curr.major + 1}.0.0.a0"

    elif spec == "minor":
        return f"{curr.major}.{curr.minor + 1}.0.a0"

    elif spec in ["a", "b", "rc"]:
        # This wasn't a pre-release before.
        if curr.pre is None:
            return f"{curr.major}.{curr.minor}.{curr.micro}.a0"

        p = curr.pre[0]
        # Increment the pre-release number, not the type of pre-release.
        if p == spec:
            return f"{curr.major}.{curr.minor}.{curr.micro}.{curr.pre[0]}{curr.pre[1] + 1}"
        
        # Reset the pre-release number ot zero if not a pre-release.
        return f"{curr.major}.{curr.minor}.{curr.micro}.{spec}0"

    # Make a full release. No more pre-tag.
    elif spec == "release":
        # If pre release, drop the pre tag.
        if curr.pre:
            return f"{curr.major}.{curr.minor}.{curr.micro}"
        # Else this is considered a patch release.
        return f"{curr.major}.{curr.minor}.{curr.micro + 1}"
    
    elif spec == "next":
        spec = f"{curr.major}.{curr.minor}."
        if curr.pre:
            p, x = curr.pre
            spec += f"{curr.micro}{p}{x + 1}"
        else:
            spec += f"{curr.micro + 1}"

    elif spec == "patch":
        spec = f"{curr.major}.{curr.minor}."
        if curr.pre:
            spec += f"{curr.micro}"
        else:
            spec += f"{curr.micro + 1}"
    else:
        raise ValueError("Unknown version spec")

    return spec


@click.command()
@click.option("--force", default=False, is_flag=True)
@click.option("--skip-if-dirty", default=False, is_flag=True)
@click.argument("spec", nargs=1)
def bump(force, skip_if_dirty, spec):
    status = run("git status --porcelain").strip()
    if len(status) > 0:
        if skip_if_dirty:
            return
        raise Exception("Must be in a clean git state with no untracked files")


    __version__ = get_version()
    version = parse(increment_version(__version__, spec))

    # convert the Python version
    js_version = f"{version.major}.{version.minor}.{version.micro}"
    if version.pre:
        p, x = version.pre
        p = p.replace("a", "alpha").replace("b", "beta")
        js_version += f"-{p}.{x}"

    # bump the JS packages
    lerna_cmd = f"{LERNA_CMD} {js_version}"
    if force:
        lerna_cmd += " --yes"
    run(lerna_cmd)

    HERE = Path(__file__).parent.parent.resolve()

    project_pins = {}

    path = HERE.joinpath("package.json")
    if path.exists():
        with path.open(mode="r") as f:
            data = json.load(f)

        data["version"] = js_version

        with path.open(mode="w") as f:
            json.dump(data, f, indent=2)

    else:
        raise FileNotFoundError(f"Could not find package.json under dir {path!s}")


if __name__ == "__main__":
    bump()